library heavylist;

import 'dart:async';

import 'resume.dart';

//used for timed foreach
class HeavyList<T> {
  List<T> _list;
  int index;
  int length;
  List<T> _registered;
  Timer t;
  //gap
  Duration _time;
  Function _completed;
  Function _each;
  bool _started = false;
  StreamController<dynamic> _removeController;
  StreamController<List<T>> _startController;
  bool _stopped = false;
  HeavyList(List<T> list) {
    this._registered = new List<T>();
    _removeController = new StreamController<dynamic>();
    _startController = new StreamController<List<T>>();
    this._list = list;
    this.index = -1;
    this.length = this._list.length;
  }

  ///Add item at a specific index
  void addItemAt(dynamic item, int index) {
    this._list.insert(index, item);
  }

  ///Add item at the end of the list
  void addItem(item) {
    this._list.add(item);
    this.length += 1;
  }

  ///Remove Specifice Item
  void removeItem(item) {
    int index = _list.indexOf(item);
    if (index >= 0) {
      _removeController.add(this._list.removeAt(index));
      this.length -= 1;
    }
  }

  ///Identifies when an item is removed
  Stream<dynamic> get clear {
    return _removeController.stream;
  }

  ///Identifies when the loop is running
  Stream<dynamic> get begun {
    return _startController.stream;
  }

  void loop(Duration time, void finished(List<T> completion),
      void each(T item, void forward())) {
    var resume, h;
    this._time = time;
    this._completed = finished;
    this._each = each;
    resume = new Resume(() {
      t = new Timer(time, h);
    });
    h = () {
      this.index += 1;

      if (this.index == 0 && !this._stopped) {
        _startController.add(this._list);
      }
      if (this.index < this.length) {
        this._registered.add(this._list[this.index]);
      }
      this._started = true;
      if (this.index == this.length) {
        _removeController.close();
        _startController.close();
      }
      this.index == this.length
          ? finished(this._list)
          : each(this._list[this.index], resume.callback);
    };

    if (!this._stopped) {
      resume.call();
    }
  }

  ///Clear the list
  void empty() {
    this.stop();
    this._list.clear();
    this.index = -1;
    this.length = 0;
  }

  ///Stop Loop
  void stop() {
    Resume.clear();
    this._started = false;
    this._stopped = true;
    if (t != null) {
      t.cancel();
      t = null;
    }
  }

  ///Pause loop
  List<T> pause() {
    if (!this._stopped) {
      this.stop();
      this._started = false;
      this.clearRegistered();
    }
    return this._list;
  }

  bool get started {
    return this._started;
  }

  void resume() {
    if (this._stopped) {
      this._stopped = false;
      this.loop(this._time, this._completed, this._each);
    }
  }

  ///Remove items already indexed
  void clearRegistered() {
    this._registered.forEach((item) {
      this._list.remove(item);
    });
    this._registered.clear();
    this.index = -1;
    this.length = this._list.length;
  }

  ///Get list
  List<T> getList() {
    return this._list;
  }

  int getSize() {
    return this._list.length;
  }
}
